/*
 * Sequence.cpp
 *
 *  Created on: Apr 15, 2010
 *      Author: Oleksiy Kohany <kohany@gmail.com>
 */

#include "Sequence.h"
#include <iostream>

namespace bio {

   const char Sequence::GAPCHAR = '-';
   Sequence::Sequence() {
   }
   Sequence::Sequence(const Sequence& other) : data(other.data) {
   }
   Sequence::Sequence(const string& data) : data(data) {
   }
   Sequence::Sequence(const char* data) : data(data) {
   }
   Sequence::~Sequence() {
   }
   Sequence& Sequence::read(istream& is) {
      clear();
      string sLine;
      if(readHeader(is)) {
         for(char c = is.peek(); is.good() && c != ';' && c != '>' && !getline(is, sLine).fail(); c = is.peek()) {
            data.append(bio::clean(sLine));
         }
      }
      else {
         is.setstate(ios::failbit);
      }
      return *this;
   }
   void Sequence::readFAHeaderLine(const string& s) {
   }
   void Sequence::readIGHeaderLine(const string& s) {
   }
   bool Sequence::readHeader(istream& is) {
      string sLine;
      char c = is.peek();
      if (is.good()) {
         switch (c) {
         case ';':
            return readIGHeader(is);
         case '>':
            return readFAHeader(is);
         }
      }
      return false;
   }
   bool Sequence::readFAHeader(istream& is) {
      if (is.peek() == '>') {
         string sLine;
         if (!getline(is, sLine).fail()) {
            readFAHeaderLine(sLine);
            return true;
         }
      }
      return false;
   }
   bool Sequence::readIGHeader(istream& is) {
      if (is.peek() == ';') {
         string sLine;
         do {
            getline(is, sLine);
            readIGHeaderLine(sLine);
         } while (!is.fail() && sLine[0] == ';');
         return true;
      }
      return false;
   }
   void Sequence::print(ostream& os, bool format) const {
      printFA(os, format);
   }
   void Sequence::printFA(ostream& os, bool format) const {
      printFAHeader(os);
      format?printFormattedBody(os):printUnformattedBody(os);
   }
   void Sequence::printIG(ostream& os, bool format) const {
      printIGHeader(os);
      format?printFormattedBody(os):printUnformattedBody(os);
   }
   void Sequence::printHeader(ostream& os) const {
      printFAHeader(os);
   }
   void Sequence::printFAHeader(ostream& os, const char* comment) const {
      os << ">ANONYMOUS";
      if(comment) {
         os << "\t" << comment;
      }
      os << endl;
   }
   void Sequence::printIGHeader(ostream& os, const char* comment) const {
      if(comment) {
         os << "; " << comment << "\n";
      }
      os << ";\nANONYMOUS\n";
   }
   void Sequence::printFormattedBody(ostream& os, size_t width) const {
      const char* sptr = data.c_str();
      for(size_t i = 0; i < data.size(); i += width) {
         os.write(data.c_str() + i, min(width, data.size() - i));
         os << "\n";
      }
   }
   void Sequence::printUnformattedBody(ostream& os) const {
      os << data << endl;
   }
   Sequence& Sequence::clean() {
      bio::clean(data);
      return *this;
   }
   Sequence& Sequence::remove(char_t c) {
      size_t pos = 0;
      while((pos = data.find(c, pos)) != data.npos) {
         data.erase(pos, 1);
      }
      return *this;
   }

   string& clean(string& s) {
      string::iterator ptr = s.begin();
      while (ptr != s.end()) {
         if (!isgraph(*ptr)) {
            ptr = s.erase(ptr);
         } else {
            ptr++;
         }
      }
      if (s.size() > 0 && *--ptr == '1') {
         s.erase(ptr);
      }
      return s;
   }
   ostream& operator<<(ostream& os, const Sequence& seq) {
      seq.print(os);
      return os;
   }
   istream& operator>>(istream& is, Sequence& seq) {
      seq.read(is);
      return is;
   }
   char complement(char c) {
      //                          "ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz"
      static const char index[] = "TVGHEFCDIJMLKNOPQYSAABWXRZ[\\]^_`tvghefcdijmlknopqysaabwxrz";
      if (c < 'A' || c > 'z') {
         return c;
      }
      return index[c - 'A'];
   }

}
