/*
 * SubSequence.h
 *
 *  Created on: Apr 15, 2010
 *      Author: Oleksiy Kohany <kohany@gmail.com>
 */

#ifndef SUBSEQUENCE_H_
#define SUBSEQUENCE_H_

#include "Sequence.h"
#include "Fragment.h"
#include <list>
#include <string.h>
using namespace std;
/*
 *
 */
namespace bio {

   class SubSequence : virtual public Sequence {
   public:
      typedef list<Fragment> fraglist_t;
   private:
      fraglist_t fragments;
   protected:
      virtual void readFAHeaderLine(const string& s);
      virtual void readIGHeaderLine(const string& s);
      void readFragments(const string& line, size_t pos = 0);
   public:
      static const char FRAGMENT[];
      SubSequence();
      SubSequence(const SubSequence& other);
      virtual ~SubSequence();
      using Sequence::operator=;
      SubSequence& operator=(const SubSequence& other) {
         Sequence::operator=(other);
         fragments = other.getFragments();
         return *this;
      }
      bool operator==(const SubSequence& other) {
         return Sequence::operator==(other) && fragments==other.fragments;
      }
      virtual void clear() {
         Sequence::clear();
         fragments.clear();
      }
      const fraglist_t& getFragments() const {
         return fragments;
      }
      void addFragment(const Fragment& f) {
         fragments.push_back(f);
      }
      void setFragment(const Fragment& f);
      void setFragment(size_t b, size_t e);
      void setBeg(size_t b);
      void setEnd(size_t e);
      size_t getBeg();
      size_t getEnd();
      bool isDirect() const;
      void reposition(const Fragment& parent);
      virtual void printFAHeader(ostream& os, const char* comment = NULL) const;
      virtual void printIGHeader(ostream& os, const char* comment = NULL) const;
   };
}

#endif /* SUBSEQUENCE_H_ */
