/*
 * IstreamWrapper.cpp
 *
 *  Created on: Mar 19, 2010
 *      Author: Oleksiy Kohany <kohany@gmail.com>
 */

#include "IOWrapper.h"
#include <iostream>
#include <stdexcept>

IstreamWrapper::IstreamWrapper() : is(&cin) {

}
IstreamWrapper::IstreamWrapper(const char* fn) : is(NULL) {
   open(fn);
}
IstreamWrapper::~IstreamWrapper() {
   close();
}
void IstreamWrapper::close() {
   if(is != NULL && is != &cin) {
      delete is;
   }
   is = NULL;
}
void IstreamWrapper::open(const char* fn) {
   if (fn != NULL) {
      close();
      if (fn[0] == '-' && fn[1] == '\0') {
         is = &cin;
         return;
      }
      is = new ifstream(fn);
      if ((is->good())) {
         return;
      }
   }
   close();
   is = &cin;
   throw invalid_argument("Failed to open a file for reading");
}
IstreamWrapper::operator istream& () {
   return *is;
}

IFstreamWrapper::IFstreamWrapper(const char* fn) : IstreamWrapper(fn) {
}
IFstreamWrapper::~IFstreamWrapper() {

}
void IFstreamWrapper::open(const char* fn) {
   if (fn != NULL) {
      close();
      is = new ifstream(fn);
      if ((is->good())) {
         return;
      }
   }
   close();
   is = &cin;
   throw invalid_argument("Failed to open a file for reading");
}

OstreamWrapper::OstreamWrapper() : os(&cout) {

}
OstreamWrapper::OstreamWrapper(const char* fn) : os(NULL) {
   open(fn);
}
OstreamWrapper::~OstreamWrapper() {
   close();
}
void OstreamWrapper::close() {
   if(os != NULL && os != &cout) {
      delete os;
   }
   os = NULL;
}
void OstreamWrapper::open(const char* fn) {
   if (fn != NULL) {
      close();
      if (fn[0] == '-' && fn[1] == '\0') {
         os = &cout;
         return;
      }
      os = new ofstream(fn);
      if ((os->good())) {
         return;
      }
   }
   close();
   os = &cout;
   throw invalid_argument("Failed to open a file for writing");
}
OstreamWrapper::operator ostream& () {
   return *os;
}
