/*
 * ThreadPool.cpp
 *
 *  Created on: Dec 1, 2009
 *      Author: kohany
 */
#include <errno.h>
#include <string.h>
#include <iostream>
#include <stdexcept>
#include <bits/local_lim.h>
//#include <ulimit.h>
#include <unistd.h>
#include "ThreadPool.h"

ThreadPool::ThreadPool() : threadCount(getCoreCount()), poolIsBusy(false), workerThreads(NULL), currentMethod(NULL), currentMethodData(NULL) {
   workerThreads = new pthread_t[threadCount];
}
ThreadPool::ThreadPool(int threadCount) : threadCount(threadCount), poolIsBusy(false), workerThreads(NULL), currentMethod(NULL), currentMethodData(NULL) {
   if(threadCount < 0 || threadCount > sysconf(_SC_THREAD_THREADS_MAX)) {
      threadCount = getCoreCount();
   }
   workerThreads = new pthread_t[threadCount];
}

ThreadPool::~ThreadPool() {
   delete [] workerThreads;
}
int ThreadPool::getThreadCount() {
   return threadCount;
}
void ThreadPool::setThreadCount(int count) {
   if(poolIsBusy) {
      cerr << "Thread pool is busy" << endl;
      throw(logic_error("Thread pool is busy."));
   }
   threadCount = count;
}
void ThreadPool::initWorkers(void *(*func)(void*), void* args, size_t stack_size) {
   if(poolIsBusy) {
      cerr << "Thread pool is busy" << endl;
      throw(logic_error("Thread pool is busy."));
   }
   pthread_attr_t tattr;
   pthread_attr_init(&tattr);
   stack_size += PTHREAD_STACK_MIN;
   pthread_attr_setstacksize(&tattr, stack_size);
   for(int i = 0; i < threadCount; i++) {
      int code = pthread_create(workerThreads + i, &tattr, func, args);
      if (code)
      {
         for(i--; i >= 0; i--) {
            pthread_cancel(workerThreads[i]);
         }
         errno = code;
         throw(runtime_error(strerror(errno)));
      }
      //cerr << "Created thread #" << i << endl;
   }
   currentMethod = (void*)func;
   currentMethodData = args;
   poolIsBusy = true;
}
void ThreadPool::joinWorkers() {
   for(int i = 0; i < threadCount; i++) {
      pthread_join(workerThreads[i], NULL);
   }
   poolIsBusy = false;
}
size_t ThreadPool::getCoreCount() {
   return sysconf(_SC_NPROCESSORS_ONLN);
}
