#include "NamedSubSequence.h"
#include "PairwiseAlignment.h"
#include "IOWrapper.h"
#include <vector>
#include <stdlib.h>
#include <math.h>
#include <sstream>
#include <boost/program_options.hpp>
namespace po = boost::program_options;


using namespace std;
using namespace bio;
typedef vector<NamedSequence> slist_t;
const char ALPHABET[] = {'A','T','C','G'};
template <typename T>
T rand(T maxval) {
   return round((double)maxval*rand()/RAND_MAX);
}
void mutate(slist_t& slist, size_t nmutations) {
   for(int i = 0; i < nmutations; i++) {
      NamedSequence& sq = slist[rand(slist.size()-1)];
      sq[rand(sq.size()-1)] = ALPHABET[rand(sizeof(ALPHABET)-1)];
   }
}
/** ======================================================== */
void usage(char* exec, po::options_description od) {
   if (!exec)
      return;
   if (strrchr(exec, '/'))
      exec = strrchr(exec, '/') + 1;
   cerr << "NAME\n\t" << exec << " - Applies given number of random mutations to randomly selected DNA sequences from the set.\n\n";
   cerr << "SYNOPSYS\n\t" << exec << " infile1(-) [options]\n";
   cerr << od;
   cerr << "\nAUTHOR\n\tOleksiy Kohany, kohany@girinst.org\n";
#ifdef COPYRIGHT

   cerr << "\nCOPYRIGHT\n\t" << COPYRIGHT << "\n";
#endif
}

/** ======================================================== */
void showVersion(const char* exec) {
   if (!exec)
      return;
   if (strrchr(exec, '/'))
      exec = strrchr(exec, '/') + 1;
   cerr << exec;
#ifdef VERSION

   cerr << "\tversion " << VERSION;
#endif

   cerr << endl;
}

int main(int argc, char** argv) {
   string infile;
   string outfile;

   double nmutations = 0;
   size_t ncopies = 1;

   po::options_description generic("GENERIC OPTIONS");
   po::options_description settings("SETTINGS");
   po::options_description hidden("HIDDEN OPTIONS");

   generic.add_options()
   ("version,v", "print version string.")
   ("help", "produce help message.")
   //("debug", "will show debugging messages on execution process.")
   ;

   settings.add_options()
   ("nc", po::value<size_t>(&ncopies), "number of copies to create of each input sequence.")
   ("mr", po::value<double>(&nmutations), "ratio of mutations - ratio of number of mutations to number of bases in the set.")
   ;

   hidden.add_options()
   ("input-file", po::value<string>(&infile), "set of sequence.")
   ("output-file", po::value<string>(&outfile)->default_value("-"), "output file. If '-' - stdout will be taken")
   ;

   po::options_description cmdline_options;
   cmdline_options.add(generic).add(settings).add(hidden);

   po::options_description visible_options;
   visible_options.add(generic).add(settings);

   po::positional_options_description p;
   p.add("input-file", 1).add("output-file", 1);

   po::variables_map vm;
   po::store(po::command_line_parser(argc, argv).options(cmdline_options).positional(p).run(), vm);
   po::notify(vm);

   if (vm.count("version")) {
      showVersion(argv[0]);
      exit(0);
   }
   if (vm.count("input-file") == 0) {
      usage(argv[0], visible_options);
      exit(0);
   }
   IstreamWrapper is(infile.c_str());
   OstreamWrapper os(outfile.c_str());
   //========================================================
   NamedSequence seq;
   slist_t slist;
   size_t nbases = 0;
   while(seq.read(is).size() > 0) {
      for(int n = 0; n < ncopies; n++) {
         slist.push_back(seq);
         if(ncopies > 1) {
            ostringstream oss;
            oss << seq.getName() << "_" << n;
            slist.back().setName(oss.str());
         }
         nbases += seq.size();
      }
   }
   if (nmutations < 0) {
      return 0;
   }
   nmutations *= nbases;
   slist_t mutants = slist;
   mutate(mutants, nmutations);
   PairwiseAlignmentStats stat;
   slist_t::const_iterator lptr1, lptr2;
   for(lptr1 = mutants.begin(), lptr2 = slist.begin(); lptr1 != mutants.end(), lptr2 != slist.end(); ++lptr1, ++lptr2) {
      os << (*lptr1);
      accumulateAlignmentStats((*lptr1).begin(), (*lptr1).end(), (*lptr2).begin(), (*lptr2).end(), (string*)NULL, (string*)NULL, PairwiseAlignment::getScoreMatrix(), stat);
   }
   // Output similarity if cout is not the output stream
   if(&(os.stream()) != &cout) {
      cout << (double)stat.iMCnt / (stat.iMCnt + stat.iMMCnt);
   }
}
